<?php
/*
Import OekakiBBS 1.32 data into an OekakiPoteto-based oekaki

http://www.NineChime.com/Wacintosh.htm
Version 1.2.1 - Last modified 11/10/2012

Supported Oekakis:
	OekakiPoteto:  v4.x - v5.1.5aa
	Wacintaki:     all to v1.5.13
	Wax Poteto:    all to v5.8.9

OekakiBBS data.txt format:
	"post \t artist \t e-mail \t www \t comment \t postdate \t IP \t host\n"
*/


// GLOBALS:
$data_file   = 'data.txt';
$erase_file  = 'erase.txt';
$export_file = 'sorted.txt';
$member_file = 'members.txt';
$post_time   = time();
$last_time   = null;

$erase_count      = array();
$pics_count       = array();
$members_count    = array();
$db_members_count = array();

$data_out = ''; // Buffer for sorted.txt

$total_pics     = 0;
$total_comments = 0;
$total_members  = 0;

// MySQL reflection
$db_155_layer = FALSE;
$db_query_func     = 'mysql_query';
$db_num_rows_func  = 'mysql_num_rows';
$db_result_func    = 'mysql_result';
$db_error_func     = 'mysql_error';
$db_close_func     = 'mysql_close';
if (file_exists ('db_layer.php')) {
	$db_155_layer = TRUE;
	$db_query_func    = 'db_query';
	$db_num_rows_func = 'db_num_rows';
	$db_result_func   = 'db_result';
	$db_error_func    = 'db_error';
	$db_close_func    = 'db_close';
}

// Max SQL INSERT errors before bomb
$max_sql_errs = 15;



//	-	-	-	-	-	-	-	-	-	-	-	-	-	-	-	-	-
// Main loop

print_header();


// Print main UI selection
if (empty ($_POST['mode'])) {
	// Print selection screen
	print_ui();
}

// Build a new data file from the OekakiBBS one
elseif ($_POST['mode'] == 'build') {
	// First, get erase file as an array
	if ($fd = fopen ($erase_file, 'r')) {
		while (!feof ($fd)) {
			$erase_count[] = (int) fgets ($fd, 32);
		}
		fclose ($fd);
	} else {
		$erase_count[] = 0;
	}


	// Open data file
	$data_out .= "Pic\tPicNum\tFileName\tPostType\tApp\tTime\tpx\tpy\tName\tEMail\tWeb\tIP\tHost\tDate\tComment\n";
	$fd = fopen($data_file, 'r');
	if (!$fd) {
		p_line ('Cannot open source file: "'.$data_file.'"');
		print_footer();
		exit ();
	}

	// Read in file, line at a time
	$got_start_number = 0;
	$start_number_diff = 0;
	while (!feof ($fd)) {
		$line = trim (fgets ($fd, 4096));

		// Skip empty lines!
		if (!empty ($line)) {

			list ($pic_number, $name, $email, $web, $comment, $date, $IP, $host) = explode ("\t", $line);

			// Fix stupid type bug with in_array()
			$pic_number = (int) $pic_number;

			// Picture numbers must start with 1!
			if (!$got_start_number) {
				$start_number_diff = 1 - $pic_number;
				$got_start_number = 1;
			}


			// Trim some stuff
			$name = trim ($name);
			$comment = trim ($comment);
			// Clean up line breaks
			$comment = preg_replace ("'<(br|BR|br \/)>'", '\n', $comment);
			// Remove auto-links & stuff (code from PHP.net).
			$comment = preg_replace ("'<[\/\!]*?[^<>]*?>'si", '', $comment);

			if (!in_array ($pic_number, $erase_count)) {
				// Check if line is post or comment
				$post_type = 0;
				$pic_number = (int) $pic_number;
				if (in_array ($pic_number, $pics_count)) {
					// Pic already counted, so it's a comment
					$post_type = 2;
					$total_comments++;
				} else {
					// New pic.  It's a post.
					$post_type = 1;
					$pics_count[] = (int) $pic_number;
					$total_pics++;
				}


				// Decode date format
				$last_time = $post_time;
				$post_time = get_oebbs_time($date);
				if ($post_time === null) {
					// Posts need to be timed so they sort correctly.
					$post_time = $last_time + 1;
				}

				// Get filetype (yuck)
				if (file_exists ('./import') && is_writable ('./import')) {
					if (file_exists ('./import/'.$pic_number.'.png')) {
						$file_type = 'png';
					} elseif (file_exists ('./import/'.$pic_number.'.jpg')) {
						$file_type = 'jpg';
					} elseif (file_exists ('./import/'.$pic_number.'.gif')) {
						$file_type = 'gif';
					} else {
						$file_type = 'png';
					}
				} else {
					p_line ('Cannot access &ldquo;import&rdquo; folder.');
					print_footer();
					exit ();
				}

				// Import format:
				// [Pic#, PicName, PostType, Applet, Time, px, py, Name, EMail, Web, IP, Host, Date, Comment]
				$data_out .=
					($pic_number + $start_number_diff)."\t"
					.$pic_number."\t"
					.$file_type."\t"
					.$post_type."\t"
					."\t"
					."\t"
					."\t"
					."\t"
					.$name."\t"
					.$email."\t"
					.$web."\t"
					.$IP."\t"
					.$host."\t"
					.$post_time."\t"
					.$comment."\n";

				// Don't forget to record names
				if (!in_array ($name, $members_count)) {
					$members_count[] = $name;
					$total_members++;
				}
			}
		}
	}

	fclose ($fd);


	// Write data file
	$write_ok = 0;
	$fd = fopen ($export_file, 'w');
	if ($fd) {
		fwrite ($fd, $data_out);
		fclose ($fd);
		$write_ok++;
	}
	$fd = fopen ($member_file, 'w');
	if ($fd) {
		foreach ($members_count as $person) {
			fwrite ($fd, $person."\n");
		}
		fclose ($fd);
		$write_ok++;
	}
	if ($write_ok != 2) {
		p_line('Could not write new data files!  Check server permissions and try again.');
		print_footer();
		exit ();
	}

echo <<<EOF
	<p>
		New data files written.
	</p>
	<p>
		Total Pictures:  $total_pics<br />
		Total Comments:  $total_comments<br />
		Total Members :  $total_members<br />
	</p>


EOF;

	print_remap_ui();
}

// Rebuild mappings for 
elseif ($_POST['mode'] == 'mapnames') {
	/*
	Find an exact match for each name, and use alternating lines for name/remap pairs
	If exact name not found, try close match (to resolve spelling issues)

	Ignore $_POST['map'] == 'guests'
	*/

	if ($_POST['map'] == 'remap') {
		$fd = fopen ('members.txt', 'r');
		if (!$fd) {
			p_line('Cannot open &ldquo;members.txt&rdquo; file.  Check permissions and build new data files again.');
			print_footer();
			exit ();
		}

		// Get member names from file (I miss Perl for this!)
		$num_oemembers = 0;
		while (!feof ($fd)) {
			// trim() needed for non-UNIX
			$line = trim (fgets ($fd, 4096));

			if (!empty ($line)) {
				$members_count[] = $line;
				$num_oemembers++;
			}
		}
		natcasesort ($members_count);
		reset ($members_count);


		// Get member names from DB
		if ($db_155_layer) {
			require ('db_layer.php');
			db_open();
		} else {
			require ('dbconn.php');
		}
		if (!$dbconn) {
			p_line('Cannot connect to database: '.$db_error_func());
			print_footer();
			exit ();
		}

		$result = $db_query_func ("SELECT usrname FROM {$OekakiPoteto_MemberPrefix}oekaki WHERE usrflags!='P'");
		$num_rows = $db_num_rows_func ($result);
		for ($i = 0; $i < $num_rows; $i++) {
			$db_members_count[] = $db_result_func ($result, $i);
		}
		$db_close_func ($dbconn);


		// OK, start HTML and do our compares
echo <<<EOF
	<h2>Remap users from your old BBS with registered members of OP / Wacintaki</h2>
	<p><b>
		This script attempts to match as many names from OekakiBBS as possible with names in your OekakiPoteto database.  You can have OekakiBBS posts remapped to any registered member on your new board.  Leave the remap field blank to have all posts inserted by that person as a guest.
	</b></p>
	<p><b>
		Examples:  &ldquo;Wac=Waccoon&rdquo; would have all OekakiBBS posts under the name Wac accreditied to the member &ldquo;Waccoon&rdquo; on your OekakiPoteto board.  &ldquo;Wac=&lt;BLANK&gt;&rdquo; will have all OekakiBBS posts inserted into OekakiPoteto as a non-registered guest named &ldquo;Wac.&rdquo;
	</b></p>
	<p><b>
		Be wary of mis-spellings.  If you have a such names, like &ldquo;Waccoon&rdquo; and &ldquo;Wacoon&rdquo;, you can simply use the same OP Name for both, and they will both be mapped to the same registered user.  Guests cannot be remapped, so you'll have to live with people screwing up their own names.  <code>;)</code>
	</b></p>

	<hr />

	<form method="post" action="import.php">
		<input type="hidden" name="mode" value="writemap" />
		<input type="hidden" name="number" value="{$num_oemembers}" />
		<input type="hidden" name="dbnumber" value="{$num_rows}" />

		<table border="1" cellpadding="10">
		<tr>
		<td style="text-align: center;">
			<b>OekakiBBS Name</b>
		</td>
		<td style="text-align: center;">
			<b>OP Name</b>
		</td>
		</tr>


EOF;

		// This is an expensive process compared to in_array(), but helps to weed out mis-spellings.
		// We'll assume that spoofers have already been banned / deleted from board.
		// We'll also allow non-alphabetical letters due to language concerns.
		$i = 0;
		foreach ($members_count as $old) {
			foreach ($db_members_count as $new) {
				$found = '';
				$checkold = mb_strtolower ($old);
				$checknew = mb_strtolower ($new);

				if ($checkold == $checknew) {
					// Match!  :)
					$found = $new;
					break;
				} else {
					$distance = levenshtein ($checkold, $checknew);

					if ($distance < 1) {
						// Will not be 0, but function may fail
						break;
					}

					// Short strings are likely to have false hits.
					if ((strlen ($new) / $distance) > 7) {
						// 1 error per 8 chars.  Good enough.  ;)
						$found = $new;
						break;
					}
				}
			}

			// Print our findings
			echo (
				'		<tr><td><input type="hidden" name="o'.$i.'" value="'.$old.'" />'.$old.'</td><td>'
				.'= <input type="text" name="n'.$i.'" value="'.$found.'" />'
				.'</td></tr>'."\n"
			);

			$i++;
		}

echo <<<EOF
		</table>
		<br />

		<input type="submit" name="submit" value="Submit Names" />
	</form>


EOF;
	} else {
		p_line('Ready to do the import!  <a href="import.php>Run this script again to continue</a>.');
	}
}

// Read remap and write new members.txt file
elseif ($_POST['mode'] == 'writemap') {
	$num_oemembers = (int) $_POST['number'];
	$num_rows = (int) $_POST['dbnumber'];

	$fp = fopen ('members.txt', 'w');
	if ($fp) {
		fwrite ($fp, "OE_Name\tBBS_Name\n");

		for ($i = 0; $i < $num_oemembers; $i++) {
			// Because I'm lazy!
			if (!empty ($_POST['n'.$i])) {
				$name2 = $_POST['n'.$i];
			} else {
				$name2 = '<BLANK>';
			}

			fwrite ($fp, $_POST['o'.$i]."\t".$name2."\n");
		}
		fclose ($fp);
	} else {
		p_line('Cannot open &ldquo;members.txt&rdquo; file.  Check permissions and build new data files again.');
		print_footer();
		exit ();
	}

	p_line('Ready to do the import!  <a href="import.php>Run this script again to continue</a>.');
}

// Perform the import using the sorted.txt and members.txt files
elseif ($_POST['mode'] == 'import') {
	// Get all files
	$remap = 1;
	$mapped_names = remap_names_from_file('members.txt');
	if ($mapped_names === FALSE) {
		p_line('Cannot open &ldquo;sorted.txt.&rdquo;');
		print_footer();
		exit ();
	} elseif ($mapped_names === 'guest') {
		$remap = 0;
	}

	if (! (file_exists ('config.php') && include ('config.php'))) {
		p_line('Cannot open OekakiPoteto config file &ldquo;config.php!&rdquo;  Check file permissions.');
		print_footer();
		exit ();
	}


	// Open database
	if ($db_155_layer) {
		require ('db_layer.php');
		db_open();
	} else {
		require ('dbconn.php');
	}
	if (!$dbconn) {
		p_line('Cannot connect to database: '.$db_error_func());
		print_footer();
		exit ();
	}


	// OK, time to start putting stuff into the DB.
	$fd = fopen ('sorted.txt', 'r');
	if (!$fd) {
		p_line('Cannot open &ldquo;sorted.txt&rdquo; file.  Check file permissions.');
		$db_close_func($dbconn);
		print_footer();
		exit ();
	}

	p_line('Beginning the import.  Please wait...');

	// Skip first line
	$line = fgets($fd, 4096);

	// Import line at a time
	$sql_err_count = 0;
	while (!feof ($fd)) {
		$line = trim (fgets ($fd, 4096));

		// Skip empty lines!
		if (!empty ($line)) {
			list ($pic, $pic_name, $file_type, $post_type, $app, $time, $px, $py, $name, $email, $web, $IP, $host, $date, $comment) = explode ("\t", $line);

			// Replace line breaks with newlines
			$comment = str_replace ('\n', "\n", $comment);

			if ($remap == 1) {
				// urlencode() is to solve PHP problems with unusual characters
				$key_name = urlencode ($name);
				$real_name = $mapped_names[$key_name];
				if (empty ($real_name)) {
					// Grr... can't remap
					$real_name = $name;
				}
			} else {
				$real_name = $name;
			}


			// Database prep
			$real_name = addslashes ($real_name);
			$comment = addslashes ($comment);
			$web = trim ($web);
			if (!empty ($web)) {
				$web = 'http://'.$web;
			}

			// See if person is in database
			$in_db = 0;
			$result = $db_query_func ("SELECT COUNT(ID) FROM {$OekakiPoteto_MemberPrefix}oekaki WHERE usrname='$real_name'");
			if ($result && ($db_result_func ($result, 0) > 0)) {
				$in_db = 1;
			}


			// OK, time to import the stuff
			//
			// Test for Wacintaki instead of OP
			$wacsql = '';
			if (isset ($CFG_version) || isset ($cfg)) {
				// Wacintaki board, at least v1.1.0
				if (isset ($useThumb) || isset ($cfg['use_thumb'])) {
					$wacsql = ", ptype = '{$file_type}'";
				}
			}

			// $resno will always be $pic, so we'll update the picture count later
			$resno = (int) $pic;
			if ($post_type == 1) {
				// Picture post
				if ($in_db) {
					$sqlres = "INSERT INTO {$OekakiPoteto_Prefix}oekakidta SET PIC_ID=$resno, usrname='$real_name', comment='$comment', postdate=FROM_UNIXTIME($date), hostname='$host', IP='$IP', edittime=0, animation=0, datatype=0, postlock=1 {$wacsql}";
				} else {
					$sqlres = "INSERT INTO {$OekakiPoteto_Prefix}oekakidta SET PIC_ID=$resno, usrname='$real_name', comment='$comment', postdate=FROM_UNIXTIME($date), hostname='$host', email='$email', url='$web', IP='$IP', edittime=0, animation=0, datatype=0, postlock=1 {$wacsql}";
				}

//				p_line('<b>POST ('.$resno.')</b> '.$sqlres);
			} elseif ($post_type == 2) {
				// Comment
				if ($in_db) {
					$sqlres = "INSERT INTO {$OekakiPoteto_Prefix}oekakicmt SET PIC_ID=$resno, usrname='$real_name', comment='$comment', postdate=FROM_UNIXTIME($date), hostname='$host', IP='$IP', postname=''";
				} else {
					$sqlres = "INSERT INTO {$OekakiPoteto_Prefix}oekakicmt SET PIC_ID=$resno, usrname='Guest', comment='$comment', postdate=FROM_UNIXTIME($date), hostname='$host', email='$email', url='$web', IP='$IP', postname='$real_name'";
				}

//				p_line('<b>COMMENT ('.$resno.')</b> '.$sqlres);
			}

			$result = $db_query_func ($sqlres);
			if (!$result) {
				// Try again, then ignore
				$result = $db_query_func ($sqlres);
				if (!$result) {
					$sql_err_count++;
				}
			}

			if ($sql_err_count > $max_sql_errs) {
				p_line('Too many SQL errors!  Check your database connection!  You\'ll need to empty the oekkaidta and oekakicmt tables.');
				$db_close_func($dbconn);
				print_footer();
				exit ();
			}
		}
	}

	// Update picture count
	$result = $db_query_func ("UPDATE {$OekakiPoteto_Prefix}oekakimisc SET miscvalue=$resno WHERE miscname='piccount'");

	$db_close_func ($dbconn);

	p_line('Finished import.  <a href="import.php>Run this script again to convert pictures</a>.');
}

// Rename picture files
elseif ($_POST['mode'] == 'rename') {
	// We need the config file for Wacintaki, to get the picture name prefix.
	if (! (file_exists ('config.php') && include ('config.php'))) {
		p_line('Cannot open OekakiPoteto database file &ldquo;config.php!&rdquo;  Check file permissions.');
		print_footer();
		exit ();
	}
	$wac = 0;
	if (isset ($useThumb) || isset ($cfg['use_thumb'])) {
		$wac = 1;
	}

	// Wacintaki 1.2+ config file hack
	if (isset ($cfg['op_pics'])) {
		$OPpics = $cfg['op_pics'];
		$OPPrefix = $cfg['op_pre'];
	}

	if (! (file_exists ($OPpics) && is_writable ($OPpics)) ) {
		p_line('Cannot access or write to &ldquo;'.$OPpics.'&rdquo; folder.  Check your permissions.');
		print_footer();
		exit ();
	}

	// Open sorted file
	$fd = fopen ('sorted.txt', 'r');
	if (!$fd) {
		p_line('Cannot open &ldquo;sorted.txt&rdquo; file.  Check file permissions.');
		print_footer();
		exit ();
	}

	// Skip first line
	$line = fgets($fd, 4096);

	while (!feof ($fd)) {
		$line = trim (fgets ($fd, 4096));

		// Skip empty lines!
		if (!empty ($line)) {
			list ($pic, $pic_name, $file_type, $post_type, $app, $time, $px, $py, $name, $email, $web, $IP, $host, $date, $comment) = explode ("\t", $line);

			$post_type = (int) $post_type;

			// Only work with posts!
			if ($post_type == 1) {
				$old_file = $pic_name.'.'.$file_type;

				if ($wac) {
					$new_file = $OPPrefix.$pic.'.'.$file_type;
				} else {
					$new_file = $pic.'.'.$file_type;
				}

				if (!empty ($_POST['move'])) {
					rename ('./import/'.$old_file, $OPpics.'/'.$new_file);
				} else {
					copy ('./import/'.$old_file, $OPpics.'/'.$new_file);
				}
			}
		}
	}

	p_line('Finished picture renaming.  Your OekakiPoteto BBS should now be ready to use.');
	p_line('');
	p_line('<b>DELETE THIS SCRIPT TO PREVENT SECURITY PROBLEMS.</b>');
	p_line('');
	p_line('Also make sure you delete the import folder, and the &ldquo;data.txt&rdquo;, &ldquo;erase.txt&rdquo;, &ldquo;sorted.txt&rdquo; and &ldquo;members.txt&rdquo; files to save server space.');
}

// All done.
print_footer();

exit ();



//	-	-	-	-	-	-	-	-	-	-	-	-	-	-	-	-	-
// Function list

function print_header() {
echo <<<EOF
<html>
<body>

EOF;
}

function print_footer() {
echo <<<EOF
</body>
</html>

EOF;
}

function p_line($in) {
	echo '<p>'.$in."</p>\n";
}

function web_line($in) {
	echo $in."<br />\n";
}

function print_ui() {
echo <<<EOF
	<h2>OekakiBBS Import</h2>
	<p>
		This script converts an OekakiBBS v1.32 board to OekakiPoteto or Wacintaki.  Tested versions of OP are <b>v4.0</b> to <b>v5.1.5aa</b>.  Tested versions of Wacintaki are <b>v1.0</b> to <b>v1.2.4</b>.  Newer vesions of both boards may be supported, but <a href="http://www.NineChime.com/products/">check first</a>.
	</p>

	<hr />

	<h2>Make sure you have done the following, in order:</h2>
	<ul>
		<li><b>Made a backup of all your OekakiBBS files and pictures!</b></li>
		<li><b>Deleted all your OekakiBBS files except &ldquo;data&rdquo, &ldquo;erase,&rdquo; and your pictures folder.</b></li>
		<li><b>Renamed your OekakiBBS data files to &ldquo;data.txt&rdquo; and &ldquo;erase.txt&rdquo;</b></li>
		<li><b>Moved all your OekakiBBS pictures into a folder named &ldquo;import&rdquo;</b></li>
		<li><b>Copied all the OekakiPoteto / Wacintaki files into place and installed the board.</b></li>
		<li><b>Made sure the member database is correct if you want to remap member names.  This script will import pictures and comments but will <em>NOT</em> modify the member list for your other boards.</b></li>
		<li><b>Made sure your OP / Wacintaki board has enough slots (&ldquo;Number of Pictures to Store&rdquo;) for all your OekakiBBS posts.</b></li>
		<li><b>Made a backup of all your OekakiBBS files and pictures (really!)</b></li>
	</ul>

	<hr />

	<h2>Pick an option.  Assumed files are &ldquo;data.txt&rdquo; and &ldquo;erase.txt&rdquo;</h2>

	<form method="post" action="import.php">
		<input type="radio" name="mode" value="" checked /> Cancel<br />
		<input type="radio" name="mode" value="build" /> Stage1: Build new data files<br />
		<br />
		<input type="radio" name="mode" value="import" /> Stage2: Perform the import into OP / Wacintaki<br />
		<br />
		<input type="radio" name="mode" value="rename" /> Stage3: Copy images to OP pictures folder<br />
		&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="move" />Move pictures instead of copy (use if you're out of server space)<br />
		<br />

		<input type="submit" name="submit" value="Go" />
	</form>
EOF;
}


function print_remap_ui() {
echo <<<EOF
	<h2>Pick an option for importing member names.  Names will not be imported into the database, yet.</h2>

	<form method="post" action="import.php">
		<input type="hidden" name="mode" value="mapnames" />

		<input type="radio" name="map" value="guests" checked /> Import all names as guests (if you have a new board)<br />
		<input type="radio" name="map" value="remap" /> Match names manually (if you have existing boards)<br />
		<br />

		<input type="submit" name="submit" value="Go" />
	</form>

EOF;
}


function get_oebbs_time($buffer) {
	// $buffer = Y/M/D (DISCARD.)  H:M:S
	// Note the double spaces!

	$buffer = trim ($buffer);
	if (empty ($buffer)) {
		return null;
	}

	// Strip excess spaces
	$buffer = str_replace ('  ', ' ', $buffer);

	// Tokenize & Seperate
	list ($day_format, $dump, $time_format) = explode (' ', $buffer);
	list ($year, $month, $day) = explode ('/', $day_format);
	list ($hour, $min, $sec)   = explode (':', $time_format);

	// Now, get our time
	$final_time = mktime (intval ($hour), intval ($min), intval ($sec), intval ($month), intval ($day), intval ($year));

	if (!$final_time) {
		return null;
	} else {
		return $final_time;
	}
}


function remap_names_from_file($filename) {
	$members_remap = array();

	$fd = fopen ($filename, 'r');
	if (!$fd) {
		return FALSE;
	} else {
		// Guest or remap mode?
		$line = trim (fgets ($fd, 4096));
		if (substr ($line, 0, 7) != 'OE_Name') {
			return 'guest';
		}

		// Read in file
		while (!feof ($fd)) {
			// trim() needed for non-UNIX
			$line = trim (fgets ($fd, 4096));

			list ($key, $value) = explode ("\t", $line);

			if ($value == '<BLANK>') {
				$value = $key;
			}

			$key = urlencode ($key);
			$members_remap[$key] = $value;
		}

		return $members_remap;
	}
}
?>